package com.bernhardriedl.projects.gameoflife.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.Properties;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JColorChooser;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JSplitPane;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.UIManager;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * FileDrop 1.1
 * by Robert Harder
 * http://iharder.sourceforge.net/current/java/filedrop/
 */

import net.iharder.dnd.FileDrop;

import com.bernhardriedl.projects.gameoflife.GameOfLife;
import com.bernhardriedl.projects.gameoflife.images.ImagesContainer;
import com.bernhardriedl.projects.gameoflife.logic.CreaturesMatrix;
import com.bernhardriedl.projects.gameoflife.tools.GOLFilter;

/**
 * Tools Library 2.10
 * https://www.bernhard-riedl.com/projects/
 */

import com.bernhardriedl.projects.tools.gui.AboutDialog;
import com.bernhardriedl.projects.tools.gui.EnhancedButton;

/**
 * Kiwi Toolkit 2.4.1
 * by Mark Lindner
 * http://freshmeat.net/projects/thekiwitoolkit/
 */

import com.hyperrealm.kiwi.ui.KFrame;
import com.hyperrealm.kiwi.ui.graph.BasicDataSample;

public class MainMenu
	extends KFrame
	implements ActionListener, Runnable, FileDrop.Listener {

	private static final long serialVersionUID = -9065329234396981554L;

	/**
	 * Constants
	 */

	public static final String PROPFILE="gameoflife.ini";

	private final String PLAY_EVOLUTE="start Evolution";
	private final String PLAY_STOP="stop Evolution";

	/**
	 * Attributes
	 */

	private double maxCreatures=0;

	/**
	 * GUI Components
	 */

	private Color colorbg=new Color(190, 213, 202);

	private CreaturesMatrixCanvas creaturesMatrixCanvas=null;

	private JToolBar toolbar=new JToolBar(GameOfLife.NICENAME);

	private JTextField textfield_dimensionX=new JTextField("15");
	private JTextField textfield_dimensionY=new JTextField("15");

	private JTextField textfield_rounds=new JTextField("50");

	private JLabel label_dimensions=new JLabel("Dimensions: ");
	private JLabel label_inDimensions=new JLabel(" x ");

	private JLabel label_rounds=new JLabel("Rounds: ");

	private EnhancedButton button_play= new EnhancedButton(this.PLAY_EVOLUTE);

	private JLabel label_delay=new JLabel("Speed: ");
	private JSlider slider_delay=new JSlider(100, 1000, 200);

	/**
	 * Menu
	 */

	private JMenuBar menubar = new JMenuBar();
	private JMenu menu_file = new JMenu();
	private JMenu menu_preferences = new JMenu();
	private JMenu menu_help = new JMenu();
	
	private JMenu submenu_examples = new JMenu();

	private JMenuItem menuitem_new = new JMenuItem();
	private JMenuItem menuitem_open = new JMenuItem();
	private JMenuItem menuitem_save = new JMenuItem();
	private JMenuItem menuitem_random= new JMenuItem();
	private JMenuItem menuitem_exit = new JMenuItem();

	private JCheckBoxMenuItem menuitem_hasBounds = new JCheckBoxMenuItem();
	private JMenuItem menuitem_color1 = new JMenuItem();
	private JMenuItem menuitem_color2 = new JMenuItem();
	private JMenuItem menuitem_color3 = new JMenuItem();
	private JMenuItem menuitem_bgcolor = new JMenuItem();
	private JMenuItem menuitem_save_preferences = new JMenuItem();

	private JMenuItem menuitem_about = new JMenuItem();
	
	private JMenuItem menuitem_examples_gliders = new JMenuItem();
	private JMenuItem menuitem_examples_glidersBig = new JMenuItem();
	private JMenuItem menuitem_examples_pulsar = new JMenuItem();

	private JPanel content=new JPanel();
	private JSplitPane inner_content=new JSplitPane(JSplitPane.VERTICAL_SPLIT);

	private boolean canceled=false;

	private JFileChooser fileChooser = new JFileChooser();

	private StatisticsPanel panel_statistics = null;

	/**
	 * Constructor
	 */

	public MainMenu() {
		try {
			this.creaturesMatrixCanvas=new CreaturesMatrixCanvas(this.getDimensionX(), this.getDimensionY());

			this.panel_statistics=new StatisticsPanel(0);

			new FileDrop(this.creaturesMatrixCanvas, this);

			/**
			 * remove not used filters
			 */

			javax.swing.filechooser.FileFilter[] filters = this.fileChooser.getChoosableFileFilters();

			for (int i = 0; i < filters.length; i++)
				this.fileChooser.removeChoosableFileFilter(filters[i]);

			this.fileChooser.addChoosableFileFilter(new GOLFilter());

			ImagesContainer.loadPics();
			
			this.setBackgroundColor(this.colorbg);
			this.loadProps();

			this.setIconImage(ImagesContainer.icon.getImage());

			this.setTitle(GameOfLife.NICENAME);

			this.makeComponents();
			this.setContentPane(content);

			this.makeMenu();
			this.setJMenuBar(menubar);

			this.getRootPane().setDefaultButton(this.button_play);

			this.setSize(750, 500);
			com.bernhardriedl.projects.tools.gui.Utils.centerWindow(this);

			this.setVisible(true);
		}

		catch (Exception e) {
			JOptionPane.showMessageDialog(this, e.getMessage());
		}
	}
	
	/**
	 * Initialize GUI components
	 * @throws Exception
	 */

	private void makeComponents() throws Exception {
		this.content.setLayout(new BorderLayout());
		this.content.setBorder(BorderFactory.createEmptyBorder(7, 7, 7, 7));

		this.slider_delay.setToolTipText("adjust speed");
		this.slider_delay.setBackground(Color.WHITE);

		this.toolbar.setBackground(Color.WHITE);

		this.toolbar.add(label_dimensions);
		this.toolbar.add(textfield_dimensionY);
		this.toolbar.add(label_inDimensions);
		this.toolbar.add(textfield_dimensionX);

		this.toolbar.add(new JLabel(" "));
		this.toolbar.addSeparator();
		this.toolbar.add(new JLabel(" "));

		this.toolbar.add(label_rounds);
		this.toolbar.add(textfield_rounds);
		
		this.toolbar.add(new JLabel(" "));
		this.toolbar.addSeparator();
		this.toolbar.add(new JLabel(" "));
		
		this.toolbar.add(label_delay);
		this.toolbar.add(slider_delay);
		
		this.toolbar.add(new JLabel(" "));
		this.toolbar.addSeparator();
		this.toolbar.add(new JLabel(" "));
		
		this.toolbar.add(button_play);

		this.panel_statistics.setBorder(BorderFactory.createEmptyBorder(10, 0, 0, 0));

		this.inner_content.setTopComponent(this.creaturesMatrixCanvas);
		this.inner_content.setBottomComponent(this.panel_statistics);
		this.inner_content.setContinuousLayout(true);
		this.inner_content.setOneTouchExpandable(true);
		this.inner_content.setDividerLocation(195);
		this.inner_content.setResizeWeight(0.5);
		this.inner_content.setBorder(BorderFactory.createEmptyBorder(10, 0, 10, 0));

		this.content.add("North", this.toolbar);
		this.content.add("Center", this.inner_content);

		this.button_play.addActionListener(this);
	}

	/**
	 * Initialize menu
	 * @throws Exception
	 */
	
	private void makeMenu() throws Exception {
		this.menuitem_new.setIcon(ImagesContainer.iconNew16);
		this.menuitem_new.setText("New");
		this.menuitem_new.addActionListener(this);
		this.menuitem_new.setAccelerator(javax.swing.KeyStroke.getKeyStroke('N', java.awt.event.KeyEvent.CTRL_MASK, false));

		this.menuitem_open.setIcon(ImagesContainer.iconOpen16);
		this.menuitem_open.setText("Open");
		this.menuitem_open.addActionListener(this);
		this.menuitem_open.setAccelerator(javax.swing.KeyStroke.getKeyStroke('O', java.awt.event.KeyEvent.CTRL_MASK, false));
		this.menuitem_open.setToolTipText("you can also use drag and drop...");

		this.submenu_examples.setText("Examples");

		this.menuitem_save.setIcon(ImagesContainer.iconSave16);
		this.menuitem_save.setText("Save");
		this.menuitem_save.addActionListener(this);
		this.menuitem_save.setAccelerator(javax.swing.KeyStroke.getKeyStroke('S', java.awt.event.KeyEvent.CTRL_MASK, false));

		this.menuitem_exit.setIcon(ImagesContainer.iconStop16);
		this.menuitem_exit.setText("Exit");
		this.menuitem_exit.addActionListener(this);
		this.menuitem_exit.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F4, java.awt.event.KeyEvent.ALT_MASK, false));

		this.menuitem_hasBounds.setState(true);
		this.menuitem_hasBounds.setText("the World has Bounds");
		this.menuitem_hasBounds.setToolTipText("out of bounds neighbours will be counted if you select no bounds!");
		this.menuitem_hasBounds.addActionListener(this);

		this.menuitem_color1.setIcon(ImagesContainer.iconPreferences16);
		this.menuitem_color1.setText("Background/Basic Color");
		this.menuitem_color1.addActionListener(this);

		this.menuitem_color2.setIcon(ImagesContainer.iconPreferences16);
		this.menuitem_color2.setText("Mid-Population Color");
		this.menuitem_color2.addActionListener(this);

		this.menuitem_color3.setIcon(ImagesContainer.iconPreferences16);
		this.menuitem_color3.setText("Crowded-Population Color");
		this.menuitem_color3.addActionListener(this);

		this.menuitem_bgcolor.setIcon(ImagesContainer.iconPreferences16);
		this.menuitem_bgcolor.setText("Background Color");
		this.menuitem_bgcolor.addActionListener(this);

		this.menuitem_save_preferences.setIcon(ImagesContainer.iconSave16);
		this.menuitem_save_preferences.setText("Save Preferences");
		this.menuitem_save_preferences.addActionListener(this);

		this.menuitem_about.setIcon(ImagesContainer.iconAbout16);
		this.menuitem_about.setText("About");
		this.menuitem_about.addActionListener(this);
		this.menuitem_about.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F1, 0, false));

		this.menuitem_examples_gliders.setIcon(ImagesContainer.icon16);
		this.menuitem_examples_gliders.setText("Gliders");
		this.menuitem_examples_gliders.addActionListener(this);

		this.menuitem_examples_glidersBig.setIcon(ImagesContainer.icon16);
		this.menuitem_examples_glidersBig.setText("Gliders (big)");
		this.menuitem_examples_glidersBig.addActionListener(this);

		this.menuitem_examples_pulsar.setIcon(ImagesContainer.icon16);
		this.menuitem_examples_pulsar.setText("Pulsar");
		this.menuitem_examples_pulsar.addActionListener(this);

		this.menuitem_random.setIcon(ImagesContainer.icon16);
		this.menuitem_random.setText("Random");
		this.menuitem_random.setToolTipText("Get Random Population");
		this.menuitem_random.addActionListener(this);

		this.menu_file.setText("File");
		this.menu_file.setMnemonic('F');
		this.menu_file.add(this.menuitem_new);
		this.menu_file.add(this.menuitem_open);
		this.menu_file.addSeparator();
		this.menu_file.add(this.submenu_examples);
		this.menu_file.addSeparator();
		this.menu_file.add(this.menuitem_save);
		this.menu_file.addSeparator();
		this.menu_file.add(this.menuitem_exit);
		
		this.submenu_examples.add(this.menuitem_examples_gliders);
		this.submenu_examples.add(this.menuitem_examples_glidersBig);
		this.submenu_examples.add(this.menuitem_examples_pulsar);
		this.submenu_examples.addSeparator();
		this.submenu_examples.add(this.menuitem_random);

		this.menu_preferences.setText("Preferences");
		this.menu_preferences.add(this.menuitem_hasBounds);
		this.menu_preferences.addSeparator();
		this.menu_preferences.add(this.menuitem_color1);
		this.menu_preferences.add(this.menuitem_color2);
		this.menu_preferences.add(this.menuitem_color3);
		this.menu_preferences.add(this.menuitem_bgcolor);
		this.menu_preferences.addSeparator();
		this.menu_preferences.add(this.menuitem_save_preferences);
		this.menu_preferences.setMnemonic('P');

		this.menu_help.setText("Help");
		this.menu_help.add(this.menuitem_about);
		this.menu_help.setMnemonic('H');

		this.menubar.add(this.menu_file);
		this.menubar.add(this.menu_preferences);
		this.menubar.add(Box.createHorizontalGlue());
		this.menubar.add(this.menu_help);
	}
	
	/**
	 * load saved properties
	 * @throws Exception
	 */
	
	public void loadProps() throws Exception {
		Properties properties=new Properties();

		File file = new File(System.getProperty("user.home").toString() + System.getProperty("file.separator").toString() + MainMenu.PROPFILE);

		if (!file.exists())
			return;

		InputStream in = new FileInputStream(file);
		properties.load(in);
		in.close();

		Integer color1_r=Integer.parseInt(properties.getProperty("color1_r"));
		Integer color1_g=Integer.parseInt(properties.getProperty("color1_g"));
		Integer color1_b=Integer.parseInt(properties.getProperty("color1_b"));
		this.creaturesMatrixCanvas.setColor1(new Color(color1_r, color1_g, color1_b));

		Integer color2_r=Integer.parseInt(properties.getProperty("color2_r"));
		Integer color2_g=Integer.parseInt(properties.getProperty("color2_g"));
		Integer color2_b=Integer.parseInt(properties.getProperty("color2_b"));
		this.creaturesMatrixCanvas.setColor2(new Color(color2_r, color2_g, color2_b));

		Integer color3_r=Integer.parseInt(properties.getProperty("color3_r"));
		Integer color3_g=Integer.parseInt(properties.getProperty("color3_g"));
		Integer color3_b=Integer.parseInt(properties.getProperty("color3_b"));
		this.creaturesMatrixCanvas.setColor3(new Color(color3_r, color3_g, color3_b));

		Integer bgcolor_r=Integer.parseInt(properties.getProperty("bgcolor_r"));
		Integer bgcolor_g=Integer.parseInt(properties.getProperty("bgcolor_g"));
		Integer bgcolor_b=Integer.parseInt(properties.getProperty("bgcolor_b"));
		this.setBackgroundColor(new Color(bgcolor_r, bgcolor_g, bgcolor_b));
	}

	/**
	 * save properties
	 */

	public void saveProps() {
		Properties properties = new Properties();

		this.setBusyCursor(true);

		try {
			properties.setProperty("color1_r", String.valueOf(this.creaturesMatrixCanvas.getColor1().getRed()));
			properties.setProperty("color1_g", String.valueOf(this.creaturesMatrixCanvas.getColor1().getGreen()));
			properties.setProperty("color1_b", String.valueOf(this.creaturesMatrixCanvas.getColor1().getBlue()));

			properties.setProperty("color2_r", String.valueOf(this.creaturesMatrixCanvas.getColor2().getRed()));
			properties.setProperty("color2_g", String.valueOf(this.creaturesMatrixCanvas.getColor2().getGreen()));
			properties.setProperty("color2_b", String.valueOf(this.creaturesMatrixCanvas.getColor2().getBlue()));

			properties.setProperty("color3_r", String.valueOf(this.creaturesMatrixCanvas.getColor3().getRed()));
			properties.setProperty("color3_g", String.valueOf(this.creaturesMatrixCanvas.getColor3().getGreen()));
			properties.setProperty("color3_b", String.valueOf(this.creaturesMatrixCanvas.getColor3().getBlue()));

			properties.setProperty("bgcolor_r", String.valueOf(this.getBackgroundColor().getRed()));
			properties.setProperty("bgcolor_g", String.valueOf(this.getBackgroundColor().getGreen()));
			properties.setProperty("bgcolor_b", String.valueOf(this.getBackgroundColor().getBlue()));

			BufferedOutputStream local_settings=new BufferedOutputStream(new FileOutputStream(System.getProperty("user.home").toString()+System.getProperty("file.separator").toString()+MainMenu.PROPFILE));

			properties.store(local_settings, GameOfLife.NICENAME+" Settings");
			local_settings.close();
		}

		catch (java.io.IOException pe) {
			this.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
			JOptionPane.showMessageDialog(this, "Error while saving the settings.", "Error!", JOptionPane.ERROR_MESSAGE);
		}

		finally {
			this.setBusyCursor(false);
		}
	}

	/**
	 * getter DimensionX
	 * @return DimensionX
	 * @throws Exception
	 */

	public int getDimensionX() throws Exception {
		return Integer.parseInt(textfield_dimensionX.getText());
	}
	
	/**
	 * getter DimensionY
	 * @return DimensionY
	 * @throws Exception
	 */

	public int getDimensionY() throws Exception {
		return Integer.parseInt(textfield_dimensionY.getText());
	}

	/**
	 * getter Rounds
	 * @return Rounds
	 * @throws Exception
	 */

	public int getRounds() throws Exception {
		return Integer.parseInt(textfield_rounds.getText());
	}
  
	/**
	 * play the Game with Random Parameters
	 * @param dimensionX int
	 * @param dimensionY int
	 * @param lifeFactor int
	 * @throws Exception
	 */

	public void buildRandom(int dimensionX, int dimensionY, int lifeFactor) throws Exception {
		this.creaturesMatrixCanvas.buildFromRandom(dimensionX, dimensionY, lifeFactor);
	}

	/**
	 * invokes the thread
	 */

	public void start() {
		this.canceled=false;
		this.button_play.setText(this.PLAY_STOP);

		new Thread(this).start();
	}

	/**
	 * the thread
	 */

	public void run() {
		try {			
			this.panel_statistics.cleanUp();
			this.panel_statistics.setMaxRounds(this.getRounds());

			/**
			 * simulate the desired rounds
			 */

			for (int i=0; i<this.getRounds(); i++) {
				this.creaturesMatrixCanvas.nextGeneration();
				
				this.panel_statistics.setRounds(i+1);

				/**
				 * update statistics
				 */

				BasicDataSample ds = new BasicDataSample();
				double creatures=this.creaturesMatrixCanvas.getPercentage();

				if (this.maxCreatures<creatures)
					this.maxCreatures=creatures;

				ds.putValue("creatures", (double) creatures);
				
				this.panel_statistics.setMaxCreatures(this.maxCreatures);
				this.panel_statistics.addDataSample(ds);

				if (this.canceled)
					break;

				Thread.sleep(slider_delay.getValue());
			}
		}

		catch (Exception e) {
			JOptionPane.showMessageDialog(this, e.getMessage());
		}

		finally {
			this.canceled=false;
			this.button_play.setText(this.PLAY_EVOLUTE);
		}
	}
	
	/**
	 * save current universe
	 * @throws Exception
	 */

	public void save() throws Exception {
		this.canceled=true;
		
		int ret = this.fileChooser.showSaveDialog(this);

		if (ret == JFileChooser.APPROVE_OPTION) {
			String ext = "." +fileChooser.getFileFilter().getDescription().substring(0, 3).toLowerCase();

			String fileName=fileChooser.getSelectedFile().getAbsolutePath();

			/**
			 * force correct extension
			 */

			if (!fileName.substring(fileName.length() - 4, fileName.length()).equalsIgnoreCase(ext))
				fileName += ext;

			File f=new File(fileName);

			if (f.exists()) {
				ret = JOptionPane.showConfirmDialog(this, "Do you want to overwrite\n"+fileName+"?", "Overwrite saved universe?", JOptionPane.YES_NO_OPTION);

				if (ret==JOptionPane.OK_OPTION)
					this.creaturesMatrixCanvas.save(fileName);
			}

			else
				this.creaturesMatrixCanvas.save(fileName);
		}
	}
	
	/**
	 * opens file-dialog and
	 * loads a saved universe
	 */
	
	public void open() throws Exception {
		this.canceled=true;
		
		int ret = this.fileChooser.showOpenDialog(this);
        	
		if (ret == JFileChooser.APPROVE_OPTION)
			this.loadUniverse(fileChooser.getSelectedFile().getAbsolutePath());
	}
	
	/**
	 * loads a saved universe
	 * @param fileName
	 * @throws Exception
	 */
	
	private void loadUniverse(String fileName) throws Exception {
		int ret = JOptionPane.showConfirmDialog(this, "Do you want to replace the current universe?", "Discard current universe?", JOptionPane.YES_NO_OPTION);

		if (ret==JOptionPane.OK_OPTION) {
			this.creaturesMatrixCanvas.buildFromFile(fileName);
			this.panel_statistics.fullCleanUp();
			this.textfield_dimensionX.setText(String.valueOf(this.creaturesMatrixCanvas.getDimensionX()));
			this.textfield_dimensionY.setText(String.valueOf(this.creaturesMatrixCanvas.getDimensionY()));
		}
	}

	private void streamUniverse(String name) throws Exception {
		this.canceled=true;
		
		int ret = JOptionPane.showConfirmDialog(this, "Do you want to load a sample universe\n instead of the current one?", "Discard current universe?", JOptionPane.YES_NO_OPTION);

		if (ret==JOptionPane.OK_OPTION) {
			this.creaturesMatrixCanvas.buildFromStream(name);
			this.panel_statistics.fullCleanUp();
			this.textfield_dimensionX.setText(String.valueOf(this.creaturesMatrixCanvas.getDimensionX()));
			this.textfield_dimensionY.setText(String.valueOf(this.creaturesMatrixCanvas.getDimensionY()));
		}
	}
	
	private void newUniverse() throws Exception {
		this.canceled=true;
		
		int ret = JOptionPane.showConfirmDialog(this, "Do you want to create an empty universe?", "Discard current universe?", JOptionPane.YES_NO_OPTION);

		if (ret==JOptionPane.OK_OPTION) {
			this.panel_statistics.fullCleanUp();
			this.creaturesMatrixCanvas.buildEmpty(this.getDimensionX(), this.getDimensionY());			
		}
	}
	
	private void randomUniverse() throws Exception {
		this.canceled=true;
		
		int ret = JOptionPane.showConfirmDialog(this, "Do you want to replace the current universe\nwith a random one?", "Discard current universe?", JOptionPane.YES_NO_OPTION);

		if (ret==JOptionPane.OK_OPTION) {
			JOptionPane lifeFactorOptionPane = new JOptionPane();
			JSlider slider = MainMenu.getLifeFactorSlider(lifeFactorOptionPane);
			
			/**
			 * taken from
			 * http://www.java2s.com/Tutorial/Java/0240__Swing/UsingJOptionPanewithaJSlider.htm
			 */
			
			lifeFactorOptionPane.setMessage(new Object[] { "Please select the probability for living creatures", slider });
			lifeFactorOptionPane.setMessageType(JOptionPane.INFORMATION_MESSAGE);
			
		    JDialog dialog = lifeFactorOptionPane.createDialog(this, "LifeFactor (in %)");
		    dialog.setVisible(true);
		    
		    int lifeFactor=50;
		    
		    try {
		    	lifeFactor=(Integer.parseInt(lifeFactorOptionPane.getInputValue().toString()));
		    }
		    catch (Exception ex) {
		    	
		    	/**
		    	 * do nothing
		    	 */
		    }
		    
			this.panel_statistics.fullCleanUp();
			this.buildRandom(this.getDimensionX(), this.getDimensionY(), lifeFactor);
		}
	}
	
	static JSlider getLifeFactorSlider(final JOptionPane optionPane) {
		JSlider slider = new JSlider();
		
		slider.setMajorTickSpacing(10);
		slider.setMinorTickSpacing(10);
		slider.setMinimum(0);
		slider.setMaximum(CreaturesMatrix.randomLimit);
		slider.setValue(50);
		slider.setPaintTicks(true);
		slider.setPaintLabels(true);
		
		ChangeListener changeListener = new ChangeListener() {
			public void stateChanged(ChangeEvent changeEvent) {
				JSlider theSlider = (JSlider) changeEvent.getSource();
				
				if (!theSlider.getValueIsAdjusting()) {
					optionPane.setInputValue(new Integer(theSlider.getValue()));
				}
			}
		};
		
		slider.addChangeListener(changeListener);
		
		return slider;
  }

  /**
	 * actionPerformed
	 * @param e ActionEvent
	 */

	public void actionPerformed(ActionEvent e) {
		try {
			if (e.getSource().equals(this.button_play)) {
				if (this.button_play.getText().equals(this.PLAY_EVOLUTE))
					this.start();
				else
					this.canceled=true;
			}

			else if (e.getSource().equals(this.menuitem_random)) {
				this.randomUniverse();
			}
			
			else if (e.getSource().equals(this.menuitem_new)) {
				this.newUniverse();
			}

			else if (e.getSource().equals(this.menuitem_open)) {
				this.open();
			}

			else if (e.getSource().equals(this.menuitem_examples_gliders)) {
				this.streamUniverse("gliders");
			}

			else if (e.getSource().equals(this.menuitem_examples_glidersBig)) {
				this.streamUniverse("gliders_big");
			}

			else if (e.getSource().equals(this.menuitem_examples_pulsar)) {
				this.streamUniverse("pulsar");
			}

			else if (e.getSource().equals(this.menuitem_save)) {
				this.save();
			}

			else if (e.getSource().equals(this.menuitem_exit)) {
				com.bernhardriedl.projects.tools.gui.Utils.exitProgramm(this);
			}

			else if (e.getSource().equals(this.menuitem_about)) {
				Vector <String> text=new Vector <String> ();

				text.addElement("The Game of Life is a cellular automaton devised");
				text.addElement("by the British mathematician John Horton Conway");
				text.addElement("in 1970. It is the best-known example of");
				text.addElement("a cellular automaton.");
				
				Vector <String> urls=new Vector <String> ();
				urls.addElement("https://en.wikipedia.org/wiki/Conway%27s_Game_of_Life");

				new AboutDialog(this, GameOfLife.NICENAME, GameOfLife.VERSION, "2006-2012", "Dr. Bernhard Riedl", text, urls, 400, 195);
			}

			else if (e.getSource().equals(this.menuitem_color1)) {
				Color newColor = JColorChooser.showDialog(this, "Choose Background & Basic Color", this.creaturesMatrixCanvas.getColor1());

				if (newColor != null)
					this.creaturesMatrixCanvas.setColor1(newColor);
			}

			else if (e.getSource().equals(this.menuitem_color2)) {
				Color newColor = JColorChooser.showDialog(this, "Choose Color for Creature (including it's neighbours) > 10 Neighbours", this.creaturesMatrixCanvas.getColor2());

				if (newColor != null)
					this.creaturesMatrixCanvas.setColor2(newColor);
			}

			else if (e.getSource().equals(this.menuitem_color3)) {
				Color newColor = JColorChooser.showDialog(this, "Choose Color for a Creature (including it's neighbours) > 25 Neighbours", this.creaturesMatrixCanvas.getColor3());

				if (newColor != null)
					this.creaturesMatrixCanvas.setColor3(newColor);
			}

			else if (e.getSource().equals(this.menuitem_bgcolor)) {
				Color newColor = JColorChooser.showDialog(this, "Choose Background-Color", this.getBackgroundColor());

				if (newColor != null)
					this.setBackgroundColor(newColor);
			}

			else if (e.getSource().equals(this.menuitem_save_preferences)) {
				this.saveProps();
			}

			else if (e.getSource().equals(this.menuitem_hasBounds)) {
				this.creaturesMatrixCanvas.setBounds(menuitem_hasBounds.getState());
			}
		}

		catch (Exception ex) {
			String error="Sorry, an error occurred.";
			if (ex.getMessage()!=null && ex.getMessage().length()>0)
				error+="\n"+ex.getMessage();
			
			JOptionPane.showMessageDialog(this, error, "Error", JOptionPane.ERROR_MESSAGE);
		}
	}

	/**
	 * setBackgroundColor
	 * @param bg Color
	 */

	public void setBackgroundColor(Color bg) {
		this.setBackground(bg);
		this.content.setBackground(bg);
		this.inner_content.setBackground(bg);
		this.panel_statistics.setBackground(bg);
		this.creaturesMatrixCanvas.setBackground(bg);
	}

	/**
	 * getBackgroundColor
	 * @return background-color
	 */

	public Color getBackgroundColor() {
		return this.getBackground();
	}

	/**
	 * setLookAndFeel
	 */

	static {

		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		}

		catch(Exception ex) {

			/**
			 * do nothing
			 */

		}
	}

	/**
	 * override processWindowEvent
	 * to allow for a 'Are you sure'-prompt
	 * @param e WindowEvent
	 */

	protected void processWindowEvent(WindowEvent e) {
		if (e.getID() == WindowEvent.WINDOW_CLOSING) {
			this.canceled=true;
			com.bernhardriedl.projects.tools.gui.Utils.exitProgramm(this);
		}
	}

	public void filesDropped(File[] files) {
		if (files.length>0)
			try {
				this.canceled=true;
				this.loadUniverse(files[0].getAbsolutePath());
			}
			catch (Exception ex) {
				JOptionPane.showMessageDialog(null, "An error occurred while loading the file!", "Error", JOptionPane.ERROR_MESSAGE);
		}
		
	}
}