package com.bernhardriedl.projects.gameoflife.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import com.bernhardriedl.projects.tools.utils.Utils;

import com.hyperrealm.kiwi.ui.graph.BasicDataSample;
import com.hyperrealm.kiwi.ui.graph.Chart;
import com.hyperrealm.kiwi.ui.graph.ChartLegend;
import com.hyperrealm.kiwi.ui.graph.ChartValue;
import com.hyperrealm.kiwi.ui.graph.LineChart;
import com.hyperrealm.kiwi.ui.model.ChartModel;
import com.hyperrealm.kiwi.ui.model.DefaultChartModel;

public class StatisticsPanel
	extends JPanel
	implements ActionListener {

	private static final long serialVersionUID = 3514343657250269850L;

	private JPanel content_north = new JPanel();

    private Chart chart;
    private ChartModel model;
    private LineChart line_chart;
    private JLabel infos=new JLabel();

    private int maxRounds;
    private int rounds;

    private double maxCreatures;

    private JComboBox combobox_showLast=new JComboBox();

    private final static int showLast=100;
    private int actualshowLast=showLast;

    public StatisticsPanel(int maxCreatures) throws Exception {
        this.maxCreatures=maxCreatures;

        this.initGraph();
        this.initGUI();
    }

    private void initGUI() {
        this.line_chart = new LineChart(chart);
        this.line_chart.setModel(model);
        this.line_chart.setPointSpacing(6);
        this.line_chart.setPointRadius(1);
        this.line_chart.setTickInterval(1);
        this.line_chart.setVisible(false);

        ChartLegend legend=new ChartLegend(chart);

        /**
         * fill combobox
         */
        
        for (int i=50;i<=200;i+=25) {
            combobox_showLast.addItem(new Integer(i));
        }

        this.combobox_showLast.addActionListener(this);
        this.combobox_showLast.setEditable(true);
        this.combobox_showLast.setSelectedItem(new Integer(showLast));

        this.content_north.setLayout(new BorderLayout());
        this.content_north.add("Center",infos);
        this.content_north.add("East",combobox_showLast);
        
        this.infos.setBorder(BorderFactory.createEmptyBorder(0, 6, 0, 0));
        
        this.content_north.setBackground(Color.WHITE);
        this.line_chart.setBackground(Color.WHITE);
        this.setBackground(Color.WHITE);

        this.setLayout(new BorderLayout());
        this.add("North", content_north);
        this.add("Center", this.line_chart);
        this.add("South", legend);
    }

    private void initGraph() {
        this.chart = new Chart("Creatures");

        this.chart.addValue(new ChartValue("creatures", "as life goes by...", Color.red));
        this.model = new DefaultChartModel(chart);
     }

    public void addDataSample(BasicDataSample basidatasample) {
    	try {
    		this.line_chart.setTickInterval(this.maxCreatures/10);

    		this.infos.setText("Rounds: "+this.rounds+"/"+this.maxRounds+" - Creatures max: "+Utils.rundeDouble(this.maxCreatures)+"%");
    		this.model.addDataSample(basidatasample);

    		if (this.model.getDataSampleCount()>0)
    			this.line_chart.setVisible(true);
    		else
    			this.line_chart.setVisible(false);

    		/**
    		 * stay @ max 200 entries
    		 */
    		
    		while (model.getDataSampleCount()>this.actualshowLast) {
    			this.model.removeDataSample(0);
    		}
    	}
    	catch (Exception ex) {
    	
    		/**
    		 * do nothing 
    		 */

    	}
    }

    private int getshowLast() {
        int maxSampleCount=showLast;
        try {
            maxSampleCount=Integer.parseInt(combobox_showLast.getSelectedItem().toString());
        }
        catch (Exception e) {
            this.combobox_showLast.setSelectedItem(new Integer(StatisticsPanel.showLast));
        }
        return maxSampleCount;
    }

    public void actionPerformed(ActionEvent e) {
        if (e.getSource().equals(this.combobox_showLast)) {
            this.actualshowLast= this.getshowLast();
        }
    }

    public void setMaxCreatures(double maxCreature) {
        this.maxCreatures=maxCreature;
    }
    
    public void setMaxRounds(int maxRounds) {
        this.maxRounds=maxRounds;
    }

    public void setRounds(int rounds) {
        this.rounds=rounds;
    }

    public void fullCleanUp() {
    	this.cleanUp();
    	this.line_chart.setVisible(false);
    }

    public void cleanUp() {
    	try {
    		this.model.clear();
    	}
    	catch (Exception ex) {
    		
    		/**
    		 * do nothing
    		 */
    	}
    	
 		this.maxCreatures=0;
		this.maxRounds=0;
		this.rounds=0;
		this.infos.setText("");
    }

}
