package com.bernhardriedl.projects.tools.gui;

import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.Icon;
import javax.swing.JButton;

public class EnhancedButton
	extends JButton
	implements MouseListener, FocusListener {

	private static final long serialVersionUID = -3217191418268766433L;
	
	/**
	 * Constructor
	 */
	
	public EnhancedButton() {
		this.styleButton();
	}
	
	/**
	 * Constructor
	 * @param text text of button
	 * */
	
	public EnhancedButton(String text) {
		super(text);
		
		this.styleButton();
	}
	
	/**
	 * Constructor
	 * @param text text of button
	 * @param icon icon of button
	 */
	
	public EnhancedButton(String text, Icon icon) {
		super(text,icon);
		
		this.styleButton();
	}
	
	/**
	 * Constructor
	 * @param icon icon of button
	 */
	
	public EnhancedButton(Icon icon) {
		super(icon);
		this.styleButton();
	}
	
	/**
	 * styleButton
	 */
	
	private void styleButton() {
		this.addMouseListener(this);
		this.addFocusListener(this);
		this.setFocusPainted(false);
		this.setBorderPainted(false);
		this.setContentAreaFilled(false);
	}
	
	/**
	 * mouseEntered - button hover effect
	 * @param m MouseEvent
	 */
	
	public void mouseEntered(MouseEvent m) {
		if (this.isEnabled())
			this.showButton();
	}
	
	/**
	 * mouseExited - fits the button into the GUI
	 * @param m MouseEvent
	 */
	
	public void mouseExited(MouseEvent m) {
		this.leaveButton();
	}
	
	/**
	 * focusGained - button hover effect
	 * @param e FocusEvent
	 * */
	
	public void focusGained(FocusEvent e) {
		this.showButton();
	}
	
	/**
	 * focusLost - fits the button into the GUI
	 * @param e FocusEvent
	 */
	
	public void focusLost(FocusEvent e) {
		leaveButton();
	}
	
	/**
	 * button hover effect
	 */
	
	private void showButton() {
		this.setBorderPainted(true);
		this.setContentAreaFilled(true);
	}
	
	/**
	 * fits button into GUI
	 */
	
	private void leaveButton() {
		this.setFocusPainted(false);
		this.setBorderPainted(false);
		this.setContentAreaFilled(false);
	}

	/**
	 * not used
	 */

	public void mouseClicked(MouseEvent m) {}
	public void mousePressed(MouseEvent m) {}
	public void mouseReleased(MouseEvent m) {}
}